@@ -104,7 +104,7 @@ def cull_flowlines(NHDPlus_paths,
                    cull_isolated=True,
                    keep_comids=None,
                    outfolder='clipped_flowlines', logger=None):
-    """Cull NHDPlus data to an area defined by an ``active_area`` polygon and
+    """Cull NHDPlus version2 data to an area defined by an ``active_area`` polygon and
     to flowlines with Arbolate sums greater than specified thresholds. Also remove
     lines that are isolated from the stream network or are missing attribute information.
 
@@ -156,8 +156,8 @@ def cull_flowlines(NHDPlus_paths,
         os.makedirs(outfolder)
         logger.statement('created {}'.format(outfolder))
 
-    if asum_thresh is not None:
-        version = '_gt{:.0f}km'.format(asum_thresh)
+    if asum_thresh is not None and asum_thresh > 0.:
+        version = f'_gt{asum_thresh:.0f}km'
     else:
         version = ''
     flowlines_files, pfvaa_files, pf_files, elevslope_files = \
@@ -224,12 +224,48 @@ def cull_flowlines(NHDPlus_paths,
                              log_time=False)
     else:
         valid_comids = original_comids.union(keep_comids)
+        # remove attribute information for COMIDs that don't have flowlines
         pfvaa = pfvaa.loc[[True if c in original_comids
                            else False for c in pfvaa.index]]
         pf = pf.loc[[True if c in original_comids
                      else False for c in pf.index]]
         elevslope = elevslope.loc[[True if c in original_comids
                                    else False for c in elevslope.index]]
+        # add attribute information for any flowlines without attributes
+        # assign an arbitrary asum of 1
+        # (user will have to manually edit these invalid COMIDs later,
+        # or in this case, the minimum stream width would be assigned)
+        pfvaa_difference = list(set(fl.COMID).difference(pfvaa.ComID))
+        if any(pfvaa_difference):
+            to_append = pd.DataFrame({
+                'ComID': pfvaa_difference,
+                'ArbolateSu': [1] * len(pfvaa_difference)}, 
+            index=pfvaa_difference)
+            pfvaa = pd.concat([pfvaa, to_append], axis=0)
+        # add any missing COMIDs to routing table
+        # fill to comids with 0s (outlet condition)
+        pf_difference = list(set(fl.COMID).difference(pf.FROMCOMID))
+        if any(pf_difference):
+            to_append = pd.DataFrame({
+                'FROMCOMID': pf_difference,
+                'TOCOMID': [0] * len(pf_difference)}, 
+            index=pf_difference)
+            pf = pd.concat([pf, to_append], axis=0)
+        # add any missing COMIDs to elevations table
+        # fill missing elevations with zeros
+        elevslope_difference = list(set(fl.COMID).difference(elevslope.COMID))
+        if any(elevslope_difference):
+            to_append = pd.DataFrame({
+                'COMID': elevslope_difference,
+                'MAXELEVSMO': [0] * len(elevslope_difference), 
+                'MELEVSMO': [0] * len(elevslope_difference)}, 
+                index=elevslope_difference)
+            elevslope = pd.concat([elevslope, to_append], axis=0)
+    
+    assert pd.api.types.is_integer_dtype(pfvaa['ComID'])
+    assert pd.api.types.is_integer_dtype(pf['FROMCOMID'])
+    assert pd.api.types.is_integer_dtype(elevslope['COMID'])
+
     fl['nhd_asum'] = pfvaa.ArbolateSu
 
     # cull by arbolate sum first
@@ -314,7 +350,7 @@ def preprocess_nhdplus(flowlines_file, pfvaa_file,
                        narwidth_shapefile=None,
                        waterbody_shapefiles=None,  # for sampling NARWidth
                        buffersize_meters=50,
-                       asum_thresh=None,
+                       asum_thresh=0.,
                        known_connections=None,
                        update_up_elevations=None,
                        update_dn_elevations=None,
@@ -323,7 +359,7 @@ def preprocess_nhdplus(flowlines_file, pfvaa_file,
                        minimum_width=1.,
                        output_length_units='meters',
                        logger=None, outfolder='output/',
-                       project_epsg=None, flowline_crs=None, dest_crs=None,
+                       flowline_crs=None, dest_crs=None,
                        ):
     """Preprocess NHDPlus data to a single DataFrame of flowlines
     that each route to no more than one flowline, with width, elevation
@@ -335,7 +371,7 @@ def preprocess_nhdplus(flowlines_file, pfvaa_file,
         Path to NHDPlus NHDFlowline shapefile. May or maybe not have been
         preprocessed by :func:`~sfrmaker.preprocessing.cull_flowlines`. The flowlines
         must be in a valid projected coorinate reference system (CRS; i.e., with units of meters),
-        or a valid projected CRS must be specified with ``project_epsg``.
+        or a valid projected CRS must be specified with ``flowline_crs``.
     pfvaa_file : str
         Path to NHDPlus PlusFlowlineVAA database (.dbf file). May or maybe not have been
         preprocessed by :func:`~sfrmaker.preprocessing.cull_flowlines`. ``ArbolateSu``
@@ -417,9 +453,6 @@ def preprocess_nhdplus(flowlines_file, pfvaa_file,
     logger : sfrmaker.logger instance, optional
         Pass an existing sfrmaker.logger instance to logger the preprocessing operations,
         by default None
-    project_epsg : int
-        EPSG code for the output CRS (e.g. 5070 for NAD83 Albers).
-        Required if the flowlines are not in a valid Projected CRS.
     flowline_crs : obj
         Coordinate reference system of the NHDPlus data. Only needed if
         the data do not have a valid ESRI projection (.prj) file.
@@ -440,7 +473,6 @@ def preprocess_nhdplus(flowlines_file, pfvaa_file,
     dest_crs : obj
         Output Coordinate reference system. Same input types
         as ``flowline_crs``.
-        By default, epsg:5070
 
     Returns
     -------
@@ -509,13 +541,13 @@ def preprocess_nhdplus(flowlines_file, pfvaa_file,
                   pf_file,
                   elevslope_file,
                   ]
-    if run_zonal_statistics:
+    if run_zonal_statistics and demfile is not None and os.path.exists(demfile):
         files_list.append(demfile)
     if narwidth_shapefile is not None:
         if waterbody_shapefiles is None:
             raise ValueError("NARWidth option ")
         else:
-            if isinstance(waterbody_shapefiles, str):
+            if isinstance(waterbody_shapefiles, str) or isinstance(waterbody_shapefiles, Path):
                 waterbody_shapefiles = [waterbody_shapefiles]
             files_list += waterbody_shapefiles
     for f in files_list:
@@ -525,6 +557,7 @@ def preprocess_nhdplus(flowlines_file, pfvaa_file,
     if logger is None:
         logger = Logger()
 
+    outfolder = Path(outfolder)
     logger.log('Preprocessing Flowlines')
 
     # read NHDPlus files into pandas dataframes
@@ -540,20 +573,20 @@ def preprocess_nhdplus(flowlines_file, pfvaa_file,
         msg = ("{} not found; flowlines must have a valid projection file."
                .format(prjfile))
         logger.lraise(msg)
-    if project_epsg is not None and dest_crs is None:
-        project_crs = get_crs(epsg=project_epsg, crs=dest_crs)
-    if flowline_crs.is_geographic:
-        if project_crs is None or project_crs.is_geographic:
-            msg = ("project_epsg for a valid Projected CRS (i.e. in units of meters)\n"
+    if dest_crs is not None:
+        project_crs = get_crs(crs=dest_crs)
+    elif project_crs is None or project_crs.is_geographic:
+            msg = ("dest_crs for a valid Projected CRS (i.e. in units of meters)\n"
                    " must be specified if flowlines are in a Geographic CRS\n"
-                   "specified project_epsg: {}".format(project_epsg))
+                   f"specified dest_crs: {dest_crs}")
             logger.lraise(msg)
+    
 
     # get bounds of flowlines
     with fiona.open(flowlines_file) as src:
         flowline_bbox = box(*src.bounds)
 
-    fl = shp2df(flowlines_file) # flowlines clipped to model area
+    fl = gpd.read_file(flowlines_file) # flowlines clipped to model area
     pfvaa = shp2df(pfvaa_file)
     pf = shp2df(pf_file)
     elevslope = shp2df(elevslope_file)
@@ -571,7 +604,8 @@ def preprocess_nhdplus(flowlines_file, pfvaa_file,
 
     # to_crs the flowlines if they are not in project_crs
     if project_crs is not None and flowline_crs is not None and project_crs != flowline_crs:
-        fl['geometry'] = project(fl.geometry, flowline_crs, project_crs)
+        #fl['geometry'] = project(fl.geometry, flowline_crs, project_crs)
+        fl.to_crs(project_crs, inplace=True)
 
     # option to reuse shapefile from previous run instead of re-running zonal statistics
     # which can take an hour for large problems
@@ -640,14 +674,20 @@ def preprocess_nhdplus(flowlines_file, pfvaa_file,
         logger.statement('Writing shapefile of buffers used to determine distributary routing...')
         flccb = fl.copy()
         flccb['geometry'] = flccb.buffpoly
+        outfile = 'flowlines_{}buffers.shp'
+        if asum_thresh > 0:
+            outfile = outfile.format(f'gt{asum_thresh:.0f}km_')
+        else:
+            outfile = outfile.format('')
         df2shp(flccb.drop('buffpoly', axis=1),
-               os.path.join(outfolder, 'flowlines_gt{:.0f}km_buffers.shp'.format(asum_thresh)),
-               index=False, epsg=project_epsg)
+               outfolder / outfile,
+               index=False, crs=dest_crs)
     else:
-        assert Path(flowline_elevations_file).exists(), \
-            ("If run_zonal_statistics=False a flowline_elevations_file produced by"
-             "a previous run of the sfrmaker.preprocessing.preprocess_nhdplus() "
-             "function is needed.")
+        if not Path(flowline_elevations_file).exists():
+            raise ValueError(
+                "If run_zonal_statistics=False a flowline_elevations_file produced by"
+                "a previous run of the sfrmaker.preprocessing.preprocess_nhdplus() "
+                "function is needed.")
         flccb = shp2df(flowline_elevations_file)
         flccb.index = flccb['COMID']
         flccb['buffpoly'] = flccb['geometry']
@@ -691,6 +731,12 @@ def preprocess_nhdplus(flowlines_file, pfvaa_file,
         txt += '{} --> {}\n'.format(k, v)
 
     logger.statement(txt)
+
+    # logger statement if no elevations are available and
+    # there is no downstream COMID with Divergence == 1
+    no_main_stem_warning = 'warning: no distributaries downstream of COMID {} '
+    no_main_stem_warning += 'with Divergence == 1. Picking the first one: {}'
+
     # dictionary of values for selecting main channel at diversions
     valid_comids = set(flcc.index)
     div_elevs = dict(zip(flcc.COMID, flcc[elevcol]))
@@ -728,9 +774,10 @@ def preprocess_nhdplus(flowlines_file, pfvaa_file,
                 # Divergence == 1 is the main stemp, Divergence == 2 is minor
                 # (see NHDPlus v2 User's Guide)
                 info = flcc.loc[tocomids_c, 'Divergence'].sort_values()
-                assert info.values[0] == 1
-                tocomids[k] = info.index[0]
-                #tocomids[k] = flcc.loc[tocomids_c, 'Divergence'].sort_values().index[0]
+                selected_tocomid = info.index[0]
+                if info.values[0] != 1:
+                    logger.statement(no_main_stem_warning.format(k, selected_tocomid))
+                tocomids[k] = selected_tocomid
             else:
                 tocomids[k] = np.array(tocomids_c)[np.nanargmin(dnelevs)]
             # secondary distributaries
@@ -872,7 +919,8 @@ def preprocess_nhdplus(flowlines_file, pfvaa_file,
                                                  b=width_from_asum_b_param,
                                                  minimum_width=minimum_width,
                                                  input_units='km', output_units=output_length_units)
-
+    flcc['width1'] = flcc.width1asum
+    flcc['width2'] = flcc.width2asum
     if narwidth_shapefile is not None:
         if not os.path.exists(narwidth_shapefile):
             raise IOError("narwidth_shapefile: {} not found!".format(narwidth_shapefile))
@@ -887,8 +935,7 @@ def preprocess_nhdplus(flowlines_file, pfvaa_file,
                         crs=project_crs,
                         output_width_units=output_length_units)
         logger.log('Sampling widths from NARWidth database')
-        flcc['width1'] = flcc.width1asum
-        flcc['width2'] = flcc.width2asum
+
         frac_narwidth = np.sum(~np.isnan(flcc.narwd_mean))/len(flcc)
         logger.statement('Flowline widths estimated from arbolate sum: {0:.1%}'.format(1-frac_narwidth), log_time=False)
         logger.statement('Flowline widths sampled from NARWidth: {0:.1%}'.format(frac_narwidth), log_time=False)
@@ -897,21 +944,25 @@ def preprocess_nhdplus(flowlines_file, pfvaa_file,
 
     if active_area is not None:
         flcc = clip_flowlines_to_polygon(flcc, active_area,
-                                         flowlines_epsg=project_epsg,
+                                         crs=dest_crs,
                                          simplify_tol=100, logger=logger)
         
     # write output files; record timestamps in logger
     logger.statement('writing output')
-    df2shp(flcc.drop('buffpoly', axis=1),
-           '{}/flowlines_gt{:.0f}km_edited.shp'.format(outfolder, asum_thresh),
-           index=False, epsg=project_epsg)
+    outfile = 'preprocessed_flowlines{}.shp'
+    if asum_thresh > 0:
+        outfile = outfile.format(f'_gt{asum_thresh:.0f}km')
+    else:
+        outfile = outfile.format('')
+    df2shp(flcc.drop('buffpoly', axis=1), outfolder / outfile,
+           index=False, crs=dest_crs)
     logger.log('Preprocessing Flowlines')
 
     return flcc
 
 
 def clip_flowlines_to_polygon(flowlines, polygon,
-                              flowlines_epsg=None, crs=None,
+                              crs=None,
                               simplify_tol=100, logger=None):
     """Clip line features in a flowlines DataFrame to polygon
     features in polygon.
@@ -957,8 +1008,11 @@ def clip_flowlines_to_polygon(flowlines, polygon,
 
     # read in the active area and to_crs to same crs as flowlines
     flowlines_crs = None
-    if flowlines_epsg is not None:
-        flowlines_crs = get_crs(epsg=flowlines_epsg, crs=crs)
+    if crs is not None:
+        flowlines_crs = get_crs(crs=crs)
+    elif isinstance(flowlines, gpd.GeoDataFrame):
+        flowlines_crs = flowlines.crs
+        
     active_area_polygon = read_polygon_feature(polygon, flowlines_crs)
 
     # simplify polygon vertices to speed intersection testing
@@ -981,7 +1035,7 @@ def clip_flowlines_to_polygon(flowlines, polygon,
 
 def sample_NARWidth(flowlines, narwidth_shapefile, waterbodies,
                     filter=None,
-                    flowlines_epsg=None, crs=None,
+                    crs=None,
                     output_width_units='meters',
                     outpath='shps/'):
     """
@@ -1038,7 +1092,7 @@ def sample_NARWidth(flowlines, narwidth_shapefile, waterbodies,
     if not os.path.isdir(outpath):
         os.makedirs(outpath)
 
-    flowline_crs = get_crs(epsg=flowlines_epsg, crs=crs)
+    flowline_crs = get_crs(crs=crs)
     if flowline_crs.is_geographic:
         msg = ("Flowlines must be in a projected Coordinate Reference System "
                "(CRS; i.e. with units of meters).")
@@ -1054,7 +1108,7 @@ def sample_NARWidth(flowlines, narwidth_shapefile, waterbodies,
     buffers = [g.buffer(buffdist) for g in flowlines.geometry]
     flbuff = flowlines.copy()
     flbuff['geometry'] = buffers
-    df2shp(flbuff, '{}/flowlines_edited_buffers_{}.shp'.format(outpath, buffdist), epsg=flowlines_epsg)
+    df2shp(flbuff, '{}/flowlines_edited_buffers_{}.shp'.format(outpath, buffdist), crs=flowline_crs)
 
     # determine which narwidth segments intersect the flowline buffers
     results = intersect_rtree(nw.geometry.tolist(), flbuff.geometry.tolist())
@@ -1120,7 +1174,7 @@ def sample_NARWidth(flowlines, narwidth_shapefile, waterbodies,
     criteria = rivers_with_widths | wbs
 
     df2shp(flowlines.loc[criteria, :], '{}/flowlines_w_sampled_narwidth_elevations.shp'.format(outpath),
-           epsg=flowlines_epsg)
+           crs=flowline_crs)
 
 
 def edit_flowlines(flowlines, config_file,
@@ -1161,7 +1215,7 @@ def edit_flowlines(flowlines, config_file,
     with open(config_file) as src:
         cfg = yaml.load(src, Loader=yaml.Loader)
 
-    if isinstance(flowlines, str):
+    if isinstance(flowlines, str) or isinstance(flowlines, Path):
         logger.log_file_and_date_modified(flowlines)
         df = shp2df(flowlines)
         # make a backup
@@ -1221,7 +1275,7 @@ def edit_flowlines(flowlines, config_file,
     #notin = set(df[toid_column]).difference(df.index)
 
     # write out an updated version of the input flowlines file
-    if isinstance(flowlines, str):
+    if isinstance(flowlines, str) or isinstance(flowlines, Path):
         df2shp(df, flowlines, prj=prj_file)
         logger.statement('wrote {}'.format(flowlines))
     return df
