@@ -590,25 +590,29 @@ class SFRData(DataPackage):
                     idomain = bas6.ibound.array
                     
             nlay = botm.shape[0] + 1
-            layers, new_botm = assign_layers(self.reach_data, botm_array=botm, idomain=idomain)
+            layers, new_botm = assign_layers(
+                self.reach_data, botm_array=botm, idomain=idomain)
             self.reach_data['k'] = layers
             if new_botm is not None:
-                outfiles = []
-                if len(new_botm.shape) == 2:
-                    write_layers = [nlay - 1]
-                else:
-                    write_layers = list(range(nlay))
-                outpath = Path(adjusted_botm_output_path)
-                for k in write_layers:
-                    outfile = outpath / f'{self.package_name}_layer_{nlay}_new_botm_elevations.dat'
-                    np.savetxt(outfile, new_botm[-1], fmt='%.2f')
-                    outfiles.append(outfile)
-                msg = ('Sfrmaker pushed some model botm elevations downward'
-                       'to accomodate streambed bottoms. New botm elevations'
-                       'written to:\n')
-                for f in outfiles:
-                    msg += f'{f}\n'
-                print(msg)
+                new_bottom_is_same = np.allclose(botm.ravel(), 
+                                                 new_botm.ravel(), rtol=0.0001)
+                if not new_bottom_is_same:
+                    outfiles = []
+                    if len(new_botm.shape) == 2:
+                        write_layers = [nlay - 1]
+                    else:
+                        write_layers = list(range(nlay))
+                    outpath = Path(adjusted_botm_output_path)
+                    for k in write_layers:
+                        outfile = outpath / f'{self.package_name}_layer_{k}_new_botm_elevations.dat'
+                        np.savetxt(outfile, new_botm[-1], fmt='%.2f')
+                        outfiles.append(outfile)
+                    msg = ('Sfrmaker pushed some model botm elevations downward '
+                        'to accomodate streambed bottoms. New botm elevations '
+                        'written to:\n')
+                    for f in outfiles:
+                        msg += f'{f}\n'
+                    print(msg)
         else:
             print('Need a model instance with a discretization package to assign layers. '
                   'A model can be assigned to the SFRData.model attribute.')
@@ -1136,7 +1140,7 @@ class SFRData(DataPackage):
         """
         # load the configuration file
         wd = os.getcwd()
-        if isinstance(config_file, str):
+        if isinstance(config_file, str) or isinstance(config_file, Path):
             # change the cwd to the config file path
             path, config_file = os.path.split(config_file)
             path = path if len(path) > 0 else '.'
